package cn.ezhizao.common.utils;

import com.itextpdf.html2pdf.ConverterProperties;
import com.itextpdf.html2pdf.HtmlConverter;
import com.itextpdf.io.font.FontProgramFactory;
import com.itextpdf.kernel.font.PdfFont;
import com.itextpdf.kernel.font.PdfFontFactory;
import com.itextpdf.kernel.geom.PageSize;
import com.itextpdf.kernel.pdf.PdfDocument;
import com.itextpdf.kernel.pdf.PdfWriter;
import com.itextpdf.layout.font.FontProvider;
import freemarker.template.Configuration;
import freemarker.template.Template;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.*;
import java.util.Map;

public class PdfUtil {

    /**
     * 获取模板内容
     * @param templateDirectory 模板文件夹
     * @param templateName      模板文件名
     * @param paramMap          模板参数
     * @return
     * @throws Exception
     */
    public static String getTemplateContent(String templateDirectory, String templateName, Map<String, Object> paramMap) throws Exception {
        Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
        try {
            configuration.setDirectoryForTemplateLoading(new File(templateDirectory));
        } catch (Exception e) {
            System.out.println("-- exception --");
        }

        Writer out = new StringWriter();
        Template template = configuration.getTemplate(templateName,"UTF-8");
        template.process(paramMap, out);
        out.flush();
        out.close();
        return out.toString();
    }
    /**
     * HTML 转 PDF
     * @param content html内容
     * @param outPath           输出pdf路径
     * @return 是否创建成功
     */
    public static boolean html2Pdf(String content, String outPath) {
        try {
            ConverterProperties converterProperties = new ConverterProperties();
            converterProperties.setCharset("UTF-8");
            FontProvider fontProvider = new FontProvider();
            fontProvider.addSystemFonts();
            converterProperties.setFontProvider(fontProvider);
            HtmlConverter.convertToPdf(content, new FileOutputStream(outPath), converterProperties);
        } catch (Exception e) {
            System.out.println("生成模板内容失败,{}"+e);
            return false;
        }
        return true;
    }
    /**
     * HTML 转 PDF
     * @param content html内容
     * @return PDF字节数组
     */
    public static byte[] html2Pdf(String content) {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            ConverterProperties converterProperties = new ConverterProperties();
            converterProperties.setCharset("UTF-8");
            FontProvider fontProvider = new FontProvider();
            PdfFont simsun = PdfFontFactory.createFont(FontProgramFactory.createFont("simsun.ttc", 1, false));
            fontProvider.addFont(simsun.getFontProgram());
            fontProvider.addSystemFonts();
            converterProperties.setFontProvider(fontProvider);
            //设置pdf纸张
            PdfWriter writer = new PdfWriter(outputStream);
            PdfDocument pdf = new PdfDocument(writer);
            pdf.setTagged();
            //设置纸张的大小，并且设置默认的pdf大小
            PageSize pageSize = new PageSize(241f / 72 * 25.4f, 140f / 72 * 25.4f);
//            PageSize pageSize = new PageSize(241f,140f);

            pdf.setDefaultPageSize(pageSize);
            HtmlConverter.convertToPdf(content,pdf,converterProperties);
        } catch (Exception e) {
            System.out.print("生成 PDF 失败,{}"+e);
        }
        return outputStream.toByteArray();
    }

    public static byte[] html3Pdf(String content) {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            ConverterProperties converterProperties = new ConverterProperties();
            converterProperties.setCharset("UTF-8");
            FontProvider fontProvider = new FontProvider();
            PdfFont simsun = PdfFontFactory.createFont(FontProgramFactory.createFont("simsun.ttc", 1, false));
            fontProvider.addFont(simsun.getFontProgram());
            fontProvider.addSystemFonts();
            converterProperties.setFontProvider(fontProvider);
            //设置pdf纸张
            PdfWriter writer = new PdfWriter(outputStream);
            PdfDocument pdf = new PdfDocument(writer);
            pdf.setTagged();
            //设置纸张的大小，并且设置默认的pdf大小
            PageSize pageSize = new PageSize(211f / 72 * 25.4f, 93f / 72 * 25.4f);
//            PageSize pageSize = new PageSize(241f,140f);

            pdf.setDefaultPageSize(pageSize);
            HtmlConverter.convertToPdf(content,pdf,converterProperties);
        } catch (Exception e) {
            System.out.print("生成 PDF 失败,{}"+e);
        }
        return outputStream.toByteArray();
    }
}