package cn.ezhizao.framework.qrCode;

import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.hutool.extra.qrcode.QrConfig;
import com.google.zxing.BarcodeFormat;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

@Service
@Slf4j
public class BarCodeDemo {
    /**
     * 图片的宽度
     */
    private static final int IMAGE_WIDTH = 1000;
    /**
     * 图片的高度(需按实际内容高度进行调整)
     */
    private static final int IMAGE_HEIGHT = 200;
    /**
     * 二维码的宽度
     */
    private static final int QR_CODE_WIDTH = 800;
    /**
     * 二维码的宽度
     */
    private static final int QR_CODE_HEIGHT = 200;

    private static final String FORMAT_NAME = "JPG";

    /**
     * @param imgLogo logo图片
     * @param title   头部标题
     * @param content 内容
     * @param footer  底部文字
     */
    public static BufferedImage createQrCode(String imgLogo, String title, String content, String footer) {
//        // 头部文字区域高度
        int titleHeight = 70;

        // 创建主模板图片
        BufferedImage image = new BufferedImage(IMAGE_WIDTH, IMAGE_HEIGHT, BufferedImage.TYPE_INT_RGB);
        Graphics2D main = image.createGraphics();
        // 设置图片的背景色
        main.setColor(Color.white); //白色
        main.fillRect(0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);

        // 动态高度
        int height = 0;

        //***************插入二维码图片***********************************************
        Graphics codePic = image.getGraphics();
        BufferedImage codeImg;
        QrConfig config = new QrConfig();
        config.setWidth(QR_CODE_WIDTH);
        config.setHeight(QR_CODE_HEIGHT);
        if (StrUtil.isNotBlank(imgLogo)) {
            config.setImg(imgLogo);
        }
        codeImg = QrCodeUtil.generate(content, BarcodeFormat.CODE_39, config);
        // 绘制二维码
        codePic.drawImage(codeImg, (IMAGE_WIDTH - QR_CODE_WIDTH) / 2, 0, QR_CODE_WIDTH, QR_CODE_HEIGHT, null);
        codePic.dispose();

        //**********************底部公司名字*********
        Graphics2D typeLeft = image.createGraphics();
        // 设置字体颜色
        typeLeft.setColor(Color.black);
        // 设置字体
        Font footerFont = new Font("宋体", Font.BOLD, 240);
        typeLeft.setFont(footerFont);
        typeLeft.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB);
        // x开始的位置：（图片宽度-字体大小*字的个数）/2
//        int startX = 600;
        int startX = 100;
        height += 990;
        typeLeft.drawString(footer, startX, 0);

        //***************插入标志图片***********************************************
//        Graphics signPic = image.getGraphics();
//        BufferedImage signImg = null;
//        try {
//            signImg = ImageIO.read(new java.io.File(imgSign));
//        } catch (Exception e) {
//        }
//
//        if (signImg != null) {
//            signPic.drawImage(signImg, 0, 130, QR_CODE_WIDTH, QR_CODE_HEIGHT, null);
//            signPic.dispose();
//        }
        // 返回image方便后续处理是生成图片还是生成base64字符串
        return image;
    }

    // 生成图片文件
    public static void createImage(BufferedImage image, String fileLocation) {
        if (image != null) {
            try {
                ImageIO.write(image, "png", new File(fileLocation));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }



    public static void main(String[] args) {
        String content = "2311199069";
        BufferedImage bufferedImage = createQrCode(null, "物资模块", content, "");
        createImage(bufferedImage, "D:/test.png");
    }
}
