package cn.ezhizao.project.business.controller;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.PdfUtil;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.common.utils.uuid.SnowflakeIdWorker;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.qrCode.BarCodeDemo;
import cn.ezhizao.framework.qrCode.QrCodeDemo;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.service.*;
import cn.ezhizao.project.system.domain.SysDept;
import cn.ezhizao.project.system.service.ISysDeptService;
import cn.hutool.core.date.DateTime;
import cn.hutool.extra.template.TemplateException;
import freemarker.template.Template;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

/**
 * 合格证Controller
 *
 * @author ezhizao
 * @date 2024-09-02
 */
@RestController
@RequestMapping("/business/certificate")
public class BizCertificateController extends BaseController {

    @Value("${file.upload.path}")
    private String uploadPath;
    @Value("${baseUrl}")
    private String baseUrl;
    @Resource
    private IBizCertificateService bizCertificateService;
    @Resource
    private IBizDayworkService bizDayworkService;
    @Resource
    private SnowflakeIdWorker snowflakeIdWorker;
    @Resource
    private IBizProductionPlanDetailService bizProductionPlanDetailService;
    @Resource
    private IBizPackageResourceGroupService bizPackageResourceGroupService;
    @Resource
    private IBizCertificateLotService bizCertificateLotService;
    @Resource
    private IBizCompanyProductPackageService bizCompanyProductPackageService;
    @Resource
    private ISysDeptService sysDeptService;
    @Resource
    private IBizProductService bizProductService;
    @Resource
    private IBizStorageRetrievalService bizStorageRetrievalService;
    @Resource
    private IBizDayworkItemService bizDayworkItemService;
    /**
     * 生成pdf
     */
    @Autowired
    private FreeMarkerConfigurer freeMarkerConfigurer;

    /**
     * 查询合格证列表
     */
    @PreAuthorize("@ss.hasPermi('business:certificate:list')")
    @GetMapping("/list")
    @Log(title = "查询合格证列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizCertificate bizCertificate) throws NoSuchFieldException, IllegalAccessException {
//        setTenantId(bizCertificate);
        startPage();
        List<BizCertificate> list = bizCertificateService.getList(bizCertificate);
        return getDataTable(list);
    }

    /**
     * 导出合格证列表
     */
    @PreAuthorize("@ss.hasPermi('business:certificate:export')")
    @Log(title = "导出合格证列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizCertificate bizCertificate) throws NoSuchFieldException, IllegalAccessException {
//        setTenantId(bizCertificate);
        List<BizCertificate> list = bizCertificateService.getList(bizCertificate);
        ExcelUtil<BizCertificate> util = new ExcelUtil<BizCertificate>(BizCertificate.class);
        util.exportExcel(response, list, "合格证数据");
    }

    /**
     * 获取合格证详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:certificate:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取合格证详细信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizCertificateService.getById(id));
    }

    /**
     * 新增合格证
     */
    @PreAuthorize("@ss.hasPermi('business:certificate:add')")
    @Log(title = "新增合格证", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizCertificate bizCertificate) throws NoSuchFieldException, IllegalAccessException {
//        setTenantId(bizCertificate);
        return toAjax(bizCertificateService.save(bizCertificate));
    }

    /**
     * 修改合格证
     */
    @PreAuthorize("@ss.hasPermi('business:certificate:edit')")
    @Log(title = "修改合格证", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizCertificate bizCertificate) throws NoSuchFieldException, IllegalAccessException {
//        setTenantId(bizCertificate);
        return toAjax(bizCertificateService.updateById(bizCertificate));
    }

    /**
     * 删除合格证
     */
    @PreAuthorize("@ss.hasPermi('business:certificate:remove')")
    @Log(title = "删除合格证", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizCertificateService.removeBatchByIds(ids));
    }

    @GetMapping("getDayworkList")
    @Log(title = "获取打印合格证列表", businessType = BusinessType.SELECT)
    public TableDataInfo getDayworkList(BizDaywork bizDaywork) throws NoSuchFieldException, IllegalAccessException {
        startPage();
        List<BizDaywork> list = bizDayworkService.getCertificateList(bizDaywork);
        return getDataTable(list);
    }
    @GetMapping("getBatchCertificateList")
    @Log(title = "获取打印合格证列表", businessType = BusinessType.SELECT)
    public TableDataInfo getBatchCertificateList(BizDaywork bizDaywork) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizDaywork);
        startPage();
        List<BizDaywork> list = bizDayworkService.getCertificateList(bizDaywork);
        if(list.isEmpty()){
            return getDataTable(new ArrayList<>());
        }
        //pruductionDetailList
        List<Long> productionPlanDetailIds = list.stream().map(BizDaywork::getProductionPlanDetailId).collect(Collectors.toList());
        List<BizProductionPlanDetail> productionPlanDetailList = bizProductionPlanDetailService.query().in("id", productionPlanDetailIds).list();
        //storageList he retrievalList
        List<Long> lotIds = list.stream().map(BizDaywork::getLotId).collect(Collectors.toList());
        List<BizStorageRetrieval> storageLists = bizStorageRetrievalService.query().in("lot_id", lotIds).list();
        List<BizStorageRetrieval> retrievalLists = bizStorageRetrievalService.query().in("retrieval_lot_id", lotIds).list();
        //packageResourceGroupList
        List<Long> packageResourceGroupIds = list.stream().map(BizDaywork::getPackageResourceGroupId).collect(Collectors.toList());
        List<BizPackageResourceGroup> packageResourceGroupList = bizPackageResourceGroupService.query().in("id", packageResourceGroupIds).list();
        for (BizDaywork item : list) {
            BizCertificate certificate = new BizCertificate();
            BizProductionPlanDetail bizProductionPlanDetail = productionPlanDetailList.stream().filter(v->v.getId().equals(item.getProductionPlanDetailId())).findFirst().orElse(null);
            //查询零存零取数量
            List<BizStorageRetrieval> storageList = storageLists.stream().filter(v->v.getLotId().equals(item.getLotId())).collect(Collectors.toList());
            long storageNum = 0;
            if (storageList.size() > 0) {
                storageNum = storageList.stream().mapToLong(BizStorageRetrieval::getStorageNum).sum();
            }
            List<BizStorageRetrieval> retrievalList = retrievalLists.stream().filter(v->v.getRetrievalLotId().equals(item.getLotId())).collect(Collectors.toList());
            long retrievalNum = 0;
            if (retrievalList.size() > 0) {
                retrievalNum = retrievalList.stream().mapToLong(BizStorageRetrieval::getStorageNum).sum();
            }
            certificate.setId(snowflakeIdWorker.nextId());
            certificate.setCompanyId(bizProductionPlanDetail.getCompanyId());
            certificate.setFurnaceNumber(item.getFurnaceNumber());
            certificate.setCustomerName(bizProductionPlanDetail.getCompanyAlias());
            certificate.setCompanyName(item.getCompanyAlias());
            certificate.setDayworkId(item.getId());
            certificate.setLotCode(item.getLotCode());
            certificate.setProductCode(bizProductionPlanDetail.getProductCode());
            certificate.setLotId(item.getLotId());
            certificate.setDrawingNumber(bizProductionPlanDetail.getDrawingNumber());
            certificate.setSpecification(bizProductionPlanDetail.getSpecification());
            certificate.setProductDescription(bizProductionPlanDetail.getProductDescription());
            certificate.setProductionPlanDetailId(bizProductionPlanDetail.getId());
            certificate.setProductId(item.getProductId());
            long result = item.getTemporaryProcessQualifiedNum() - storageNum + retrievalNum;
            certificate.setNumber((int) result);
            certificate.setPackageNumber(item.getTemporaryProcessQualifiedNum());
            certificate.setTenantId(getLoginUser().getTenantId());
            certificate.setPrintDate(new Date());
            certificate.setFactoryName("德迈仕");
            if (item.getPackageResourceGroupId() != null && !item.getPackageResourceGroupId().equals(0L)) {
                BizPackageResourceGroup bizPackageResourceGroup = packageResourceGroupList.stream().filter(v->v.getId().equals(item.getPackageResourceGroupId())).findFirst().orElse(null);
                SysDept dept = sysDeptService.selectDeptById(bizPackageResourceGroup.getDeptId());
                certificate.setDeptName(dept.getDeptName());
                certificate.setDeptId(bizPackageResourceGroup.getDeptId());
                certificate.setPackageLineNo(bizPackageResourceGroup.getPackageLineNo().toString());
                certificate.setPackageResourceGroupId(bizPackageResourceGroup.getId());
                certificate.setPackageResourceGroupName(bizPackageResourceGroup.getName());
                certificate.setDeptNo(bizPackageResourceGroup.getDeptNo().toString());
                certificate.setYear(bizPackageResourceGroup.getYear());
            }
            BizCompanyProductPackage packageConditions = new BizCompanyProductPackage();
            packageConditions.setProductId(certificate.getProductId());
            packageConditions.setCompanyId(certificate.getCompanyId());
            List<BizCompanyProductPackage> bizCompanyProductPackages = bizCompanyProductPackageService.getList(packageConditions);
            if (bizCompanyProductPackages.size() == 1) {
                certificate.setPackageNo(bizCompanyProductPackages.get(0).getPackageNo());
                certificate.setPackageCode(bizCompanyProductPackages.get(0).getPackageCode());
                certificate.setPackageStandard(bizCompanyProductPackages.get(0).getPackageStandard());
                certificate.setPackageId(bizCompanyProductPackages.get(0).getAccessoriesId());
                certificate.setPackageTypeCode(bizCompanyProductPackages.get(0).getPackageTypeCode());
                certificate.setPurchaseOrderNo(bizCompanyProductPackages.get(0).getPurchaseOrderNo());
            }
            item.setCertificate(certificate);
        }
        return getDataTable(list);
    }

    // 生成二维码和条形码
    @PostMapping("/generateSingle")
    @Log(title = "生成二维码和条形码", businessType = BusinessType.UPDATE)
    @Transactional()
    public AjaxResult generateSingle(@RequestBody BizCertificate bizCertificate) throws NoSuchFieldException, IllegalAccessException {
        String barcode = "2311199069";
        String qrcode = barcode + "!" + bizCertificate.getLotCode() + "!" + bizCertificate.getProductCode() + "!" + bizCertificate.getDrawingNumber() + "!" + bizCertificate.getSpecification() + "!" + bizCertificate.getLabelNumber() + "!" + bizCertificate.getPackageTypeCode() + "!" + (new DateTime()).toString("yyyy-MM-dd") + "!!";

        bizCertificate = generateQrCode(qrcode, barcode, bizCertificate);
        bizCertificate = generateBarCode(barcode, barcode, bizCertificate);
        return success(bizCertificateService.saveOrUpdate(bizCertificate));
    }

    /**
     * 生成二维码和条形码，查询对应的包装线，将要存入二维码中的数据拼接成字符串，生成对应的二维码，保存到数据库中
     * @param bizCertificate
     * @return
     */
    @PostMapping("/generate")
    @Log(title = "生成二维码和条形码", businessType = BusinessType.UPDATE)
    @Transactional()
    public AjaxResult generate(@RequestBody BizCertificate bizCertificate) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizCertificate);
        BizPackageResourceGroup bizPackageResourceGroup = bizPackageResourceGroupService.getById(bizCertificate.getPackageResourceGroupId());
        int currentLineNumber = Integer.parseInt(bizPackageResourceGroup.getCode());
        List<BizCertificateLot> lots = new ArrayList<>();
        List<BizProduct> products =bizProductService.query().eq("product_code",bizCertificate.getProductCode()).list();
        bizCertificate.setMaterialNum(!products.isEmpty()?products.get(0).getMaterialNum():" ");
        for (int i = 0; i < bizCertificate.getPrintPages(); i++) {
            String barcode = bizPackageResourceGroup.getYear() + bizCertificate.getDeptNo() + bizPackageResourceGroup.getPackageLineNo() + String.format("%06d", (++currentLineNumber));
            String qrcode = barcode + "!" + bizCertificate.getLotCode() + "!" + bizCertificate.getProductCode() + "!" + bizCertificate.getDrawingNumber() + "!" + bizCertificate.getSpecification() + "!" + bizCertificate.getLabelNumber() + "!" + bizCertificate.getPackageTypeCode() + "!" + (new DateTime()).toString("yyyy-MM-dd") + "!!";
            BizCertificateLot lot = new BizCertificateLot();
            lot.setCertificateId(bizCertificate.getId());
            lot.setBarcode(barcode);
            lot.setQrcode(qrcode);
            lot.setTenantId(bizCertificate.getTenantId());
            lot.setCompanyName(bizCertificate.getCompanyName());
            lot = generateQrCode(qrcode, barcode, lot, bizCertificate);
            lot = generateBarCode(barcode, barcode, lot, bizCertificate);
            if (i == 0) {
                bizCertificate.setStartCode(barcode);
            }
            if (i == (bizCertificate.getPrintPages() - 1)) {
                bizCertificate.setEndCode(barcode);
            }
            lots.add(lot);
        }
        bizCertificate.setGenerateTime(new Date());
        bizPackageResourceGroup.setCode(String.format("%06d", currentLineNumber));
        bizPackageResourceGroupService.updateById(bizPackageResourceGroup);
        bizCertificateService.saveOrUpdate(bizCertificate);
        bizCertificateLotService.saveBatch(lots);
        bizCertificate.setLots(lots);

        return success(bizCertificate);
    }


    BizCertificate generateQrCode(String content, String fileName, BizCertificate bizCertificate) {
        BufferedImage qrImage = QrCodeDemo.createQrCode(null, "二维码", content, "");
        String dir = uploadPath + "qcCode/certificate/" + bizCertificate.getId() +
                "/qrcode/";
        // 判断文件夹是否存在，假设不存在创建你文件夹
        File file = new File(dir);
        if (!file.exists()) {
            file.mkdirs();
        }
        String path = uploadPath + "qcCode/certificate/" + bizCertificate.getId() +
                "/qrcode/" + fileName + ".png";
        bizCertificate.setQrcodeUrl("/profile/upload/qcCode/certificate/" + bizCertificate.getId() +
                "/qrcode/" + fileName + ".png");
        QrCodeDemo.createImage(qrImage, path);
        return bizCertificate;
    }


    BizCertificate generateBarCode(String content, String fileName, BizCertificate bizCertificate) {
        String dir = uploadPath + "qcCode/certificate/" + bizCertificate.getId() +
                "/barcode/";
        // 判断文件夹是否存在，假设不存在创建你文件夹
        File file = new File(dir);
        if (!file.exists()) {
            file.mkdirs();
        }
        String barPatch = uploadPath + "qcCode/certificate/" + bizCertificate.getId() + "/barcode/" + fileName + ".png";
        BufferedImage barImage = BarCodeDemo.createQrCode(null, "条形码", content, "");
        bizCertificate.setBarcodeUrl("/profile/upload/qcCode/certificate/" + bizCertificate.getId() +
                "/barcode/" + fileName + ".png");
        BarCodeDemo.createImage(barImage, barPatch);
        return bizCertificate;
    }

    BizCertificateLot generateQrCode(String content, String fileName, BizCertificateLot lot, BizCertificate bizCertificate) {
        BufferedImage qrImage = QrCodeDemo.createQrCode(null, "二维码", content, "");
        String dir = uploadPath + "qcCode/certificate/" + bizCertificate.getId() +
                "/qrcode/";
        // 判断文件夹是否存在，假设不存在创建你文件夹
        File file = new File(dir);
        if (!file.exists()) {
            file.mkdirs();
        }
        String path = uploadPath + "qcCode/certificate/" + bizCertificate.getId() +
                "/qrcode/" + fileName + ".png";
        lot.setQrcodeUrl("/profile/upload/qcCode/certificate/" + bizCertificate.getId() +
                "/qrcode/" + fileName + ".png");
        QrCodeDemo.createImage(qrImage, path);
        return lot;
    }

    BizCertificateLot generateBarCode(String content, String fileName, BizCertificateLot lot, BizCertificate bizCertificate) {
        String dir = uploadPath + "qcCode/certificate/" + bizCertificate.getId() +
                "/barcode/";
        // 判断文件夹是否存在，假设不存在创建你文件夹
        File file = new File(dir);
        if (!file.exists()) {
            file.mkdirs();
        }
        String barPatch = uploadPath + "qcCode/certificate/" + bizCertificate.getId() + "/barcode/" + fileName + ".png";
        BufferedImage barImage = BarCodeDemo.createQrCode(null, "条形码", content, "");
        lot.setBarcodeUrl("/profile/upload/qcCode/certificate/" + bizCertificate.getId() +
                "/barcode/" + fileName + ".png");
        BarCodeDemo.createImage(barImage, barPatch);
        return lot;
    }

    /**
     * 自测使用，服务器不适用此方法
     * @param bizCertificate
     * @param response
     * @throws IOException
     * @throws TemplateException
     * @throws freemarker.template.TemplateException
     */
    @Log(title = "合格证打印", businessType = BusinessType.EXPORT)
    @PostMapping("/printPdfSingle")
    public void printPdfSingle(BizCertificate bizCertificate, HttpServletResponse response) throws IOException, TemplateException, freemarker.template.TemplateException {
        List<BizCertificate> lots = new ArrayList<>();
        Map<String, Object> paramMap = new HashMap<>();
        //cdms.bmp位图（黑白）  logo.png图片(彩)
        paramMap.put("logo", baseUrl + "/profile/logo/" + "cdms.bmp");
        paramMap.put("lots", lots);
        bizCertificate.setQrcodeUrl2Src(baseUrl + bizCertificate.getQrcodeUrl());
        bizCertificate.setBarcodeUrl2Src(baseUrl + bizCertificate.getBarcodeUrl());
        lots.add(bizCertificate);
        Writer out = new StringWriter();
        // 获取模板地址
        Template template = freeMarkerConfigurer.getConfiguration().getTemplate("certificate.html");
        template.process(paramMap, out);
        out.flush();
        out.close();
        String templateContent = out.toString();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/pdf");
        String fileName = "合格证";
        response.setHeader("Content-Disposition", "filename=" + new String(fileName.getBytes()));
        byte[] resources = PdfUtil.html2Pdf(templateContent);
        ServletOutputStream outputStream = response.getOutputStream();
        outputStream.write(resources);
        outputStream.close();
    }

    @Log(title = "合格证打印", businessType = BusinessType.EXPORT)
    @PostMapping("/printPdf")
    @Transactional
    public void printPdf(BizCertificate bizCertificate, HttpServletResponse response) throws IOException, TemplateException, freemarker.template.TemplateException {

        //打印标准合格证
        if (bizCertificate.getType().equals("certificate")) {
            certificatePdf(bizCertificate, response);
        } else if(bizCertificate.getType().equals("text")){
            throw new RuntimeException("text合格证");
        }else  {
            throw new RuntimeException("合格证种类错误，请联系管理员");
        }


//        历史版本注释 *勿动*
//        List<BizCertificateLot> lots = bizCertificateLotService.query().eq("certificate_id", bizCertificate.getId()).list();
//        lots.forEach(l -> {
//            l.setQrcodeUrl2Src(baseUrl + l.getQrcodeUrl());
//            l.setBarcodeUrl2Src(baseUrl + l.getBarcodeUrl());
//        });
//        Map<String, Object> paramMap = new HashMap<>();
//        //cdms.bmp位图（黑白）  logo.png图片(彩)
//        paramMap.put("logo", baseUrl + "/profile/logo/" + "cdms.bmp");
//        paramMap.put("lots", lots);
//        paramMap.put("certificate", bizCertificate);
//        bizCertificate.setQrcodeUrl2Src(baseUrl + bizCertificate.getQrcodeUrl());
//        bizCertificate.setBarcodeUrl2Src(baseUrl + bizCertificate.getBarcodeUrl());
//        bizCertificate.setPrintTime(new Date());
////        lots.add(bizCertificate);
//        Writer out = new StringWriter();
//        // 获取模板地址
//        Template template = freeMarkerConfigurer.getConfiguration().getTemplate("certificate.html");
//        template.process(paramMap, out);
//        out.flush();
//        out.close();
//        String templateContent = out.toString();
//        response.setCharacterEncoding("UTF-8");
//        response.setContentType("application/pdf");
//        String fileName = "合格证";
//        response.setHeader("Content-Disposition", "filename=" + new String(fileName.getBytes()));
//        byte[] resources = PdfUtil.html2Pdf(templateContent);
//        bizCertificateService.updateById(bizCertificate);
//        ServletOutputStream outputStream = response.getOutputStream();
//        outputStream.write(resources);
//        outputStream.close();

    }


    @Log(title = "合格证导出", businessType = BusinessType.EXPORT)
    @PostMapping("/batchExport")
    public void batchExport(HttpServletResponse response,BizCertificate bizCertificate) throws NoSuchFieldException, IllegalAccessException, IOException {
        List<BizCertificate> certificateList = bizCertificateService.query().in("id", bizCertificate.getIds()).list();
        //存打印时间
        certificateList.forEach(v -> v.setPrintTime(new Date()));
        bizCertificateService.updateBatchById(certificateList);
        //创建时间
        List<Long> dayworkIds = certificateList.stream().map(BizCertificate::getDayworkId).collect(Collectors.toList());
        List<BizDayworkItem> dayworkItemList = bizDayworkItemService.query().in("daywork_id", dayworkIds).list();
        List<BizCertificateLot> certificateLotList = bizCertificateLotService.query().in("certificate_id", bizCertificate.getIds()).list();
        //包装详情
        List<Long> packageIds = certificateList.stream().map(v -> v.getPackageId()).distinct().collect(Collectors.toList());
        List<BizCompanyProductPackage> packageList = bizCompanyProductPackageService.query().in("id", packageIds).list();
        List<BizCertificateExport> list = new ArrayList<>();
        for(BizCertificate certificate : certificateList){
            BizDayworkItem bizDayworkItem = dayworkItemList.stream().filter(v -> v.getDayworkId().equals(certificate.getDayworkId())).sorted(Comparator.comparing(BizDayworkItem::getCreateTime).reversed()).findFirst().orElse(null);
            List<BizCertificateLot> certificateLots = certificateLotList.stream().filter(v -> v.getCertificateId().equals(certificate.getId())).collect(Collectors.toList());
            for(BizCertificateLot item : certificateLots){
                BizCertificateExport bizCertificateExport = new BizCertificateExport();
                bizCertificateExport.setLotCode(certificate.getLotCode());
                bizCertificateExport.setProductDescription(certificate.getProductDescription());
                bizCertificateExport.setDeptName(certificate.getDeptName());
                bizCertificateExport.setLabelNumber(certificate.getLabelNumber());
                bizCertificateExport.setPackageLineNo(certificate.getPackageLineNo());
                bizCertificateExport.setPackageNo(certificate.getPackageNo());
                bizCertificateExport.setNumber(certificate.getNumber());
                bizCertificateExport.setPrintPages(certificate.getPrintPages());
                bizCertificateExport.setPrintDate(certificate.getPrintDate());
                bizCertificateExport.setCompanyName(certificate.getCompanyName());
                bizCertificateExport.setPurchaseOrderNo(certificate.getPurchaseOrderNo());
                bizCertificateExport.setFurnaceNumber(certificate.getFurnaceNumber());
                bizCertificateExport.setPackageStandard(certificate.getPackageStandard());
                bizCertificateExport.setPackageTypeCode(certificate.getPackageTypeCode());
                bizCertificateExport.setSpecification(certificate.getSpecification());
                bizCertificateExport.setDrawingNumber(certificate.getDrawingNumber());
                bizCertificateExport.setMaterialNum(certificate.getMaterialNum());
                bizCertificateExport.setSubmitTime(bizDayworkItem.getEndTime());
                bizCertificateExport.setBarCode(item.getBarcode());
                bizCertificateExport.setFirst_1(certificate.getFirst_1());
                bizCertificateExport.setFirst_2(certificate.getFirst_2());
                bizCertificateExport.setFirst_3(certificate.getFirst_3());
                bizCertificateExport.setFirst_4(certificate.getFirst_4());
                bizCertificateExport.setFirst_5(certificate.getFirst_5());
                bizCertificateExport.setFirst_6(certificate.getFirst_6());
                bizCertificateExport.setFirst_7(certificate.getFirst_7());
                bizCertificateExport.setFirst_8(certificate.getFirst_8());
                bizCertificateExport.setFirst_9(certificate.getFirst_9());
                bizCertificateExport.setFirst_10(certificate.getFirst_10());
                //包装
                BizCompanyProductPackage packageItem = packageList.stream().filter(v -> v.getId().equals(certificate.getPackageId())).findFirst().orElse(null);
                bizCertificateExport.setRecipientName(packageItem.getRecipientName());
                bizCertificateExport.setPackingOrder(packageItem.getPackingOrder());
                bizCertificateExport.setCustomerPartNo(packageItem.getCustomerPartNo());
                bizCertificateExport.setCustomerPartName(packageItem.getCustomerPartName());
                bizCertificateExport.setProductCode(packageItem.getProductCode());
                bizCertificateExport.setNetWeight(packageItem.getNetWeight());
                bizCertificateExport.setSingleGrossWeight(packageItem.getSingleGrossWeight());
                bizCertificateExport.setEngineerChangeNumber(packageItem.getEngineerChangeNumber());
                bizCertificateExport.setSupplierCode(packageItem.getSupplierCode());
                list.add(bizCertificateExport);
            }
        }
        ExcelUtil<BizCertificateExport> util = new ExcelUtil<BizCertificateExport>(BizCertificateExport.class);
        util.exportExcel(response, list, "合格证打印");
    }
    /**
     * 打印标准合格证
     *
     * @param bizCertificate
     * @param response
     * @throws freemarker.template.TemplateException
     * @throws IOException
     */
    private void certificatePdf(BizCertificate bizCertificate, HttpServletResponse response) throws freemarker.template.TemplateException, IOException {
        List<BizCertificateLot> lots = bizCertificateLotService.query().eq("certificate_id", bizCertificate.getId()).list();
        lots.forEach(l -> {
            l.setQrcodeUrl2Src(baseUrl + l.getQrcodeUrl());
            l.setBarcodeUrl2Src(baseUrl + l.getBarcodeUrl());
        });
        Map<String, Object> paramMap = new HashMap<>();
        //cdms.bmp位图（黑白）  logo.png图片(彩)
        paramMap.put("logo", baseUrl + "/profile/logo/" + "cdms.bmp");
        paramMap.put("lots", lots);
        paramMap.put("certificate", bizCertificate);
        bizCertificate.setQrcodeUrl2Src(baseUrl + bizCertificate.getQrcodeUrl());
        bizCertificate.setBarcodeUrl2Src(baseUrl + bizCertificate.getBarcodeUrl());
        bizCertificate.setPrintTime(new Date());
//        lots.add(bizCertificate);
        Writer out = new StringWriter();
        // 获取模板地址
        Template template = freeMarkerConfigurer.getConfiguration().getTemplate("certificate.html");
        template.process(paramMap, out);
        out.flush();
        out.close();
        String templateContent = out.toString();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/pdf");
        String fileName = "合格证";
        response.setHeader("Content-Disposition", "filename=" + new String(fileName.getBytes()));
        byte[] resources = PdfUtil.html2Pdf(templateContent);
        bizCertificateService.updateById(bizCertificate);
        ServletOutputStream outputStream = response.getOutputStream();
        outputStream.write(resources);
        outputStream.close();

    }

    @Log(title = "新建合格证", businessType = BusinessType.SELECT)
    @PostMapping("/newCertificate")
    public AjaxResult newCertificate(@RequestBody BizDaywork bizDaywork) {
        // 获取BizDaywork
        BizCertificate certificate = new BizCertificate();
        BizProductionPlanDetail bizProductionPlanDetail = bizProductionPlanDetailService.getById(bizDaywork.getProductionPlanDetailId());
        BizProduct product = bizProductService.query().eq("id", bizProductionPlanDetail.getProductId()).one();
        //查询零存零取数量
        List<BizStorageRetrieval> storageList = bizStorageRetrievalService.query().eq("lot_id", bizDaywork.getLotId()).list();
        long storageNum = 0;
        if(storageList.size()>0){
            storageNum = storageList.stream().mapToLong(BizStorageRetrieval::getStorageNum).sum();
        }
        List<BizStorageRetrieval> retrievalList = bizStorageRetrievalService.query().eq("retrieval_lot_id", bizDaywork.getLotId()).list();
        long retrievalNum = 0;
        if(retrievalList.size()>0){
            retrievalNum = retrievalList.stream().mapToLong(BizStorageRetrieval::getStorageNum).sum();
        }
        certificate.setId(snowflakeIdWorker.nextId());
        certificate.setCompanyId(bizProductionPlanDetail.getCompanyId());
        certificate.setCustomerName(bizProductionPlanDetail.getCompanyAlias());
        certificate.setCompanyName(product.getCompanyAlias());
        certificate.setDayworkId(bizDaywork.getId());
        certificate.setLotCode(bizDaywork.getLotCode());
        certificate.setProductCode(bizProductionPlanDetail.getProductCode());
        certificate.setLotId(bizDaywork.getLotId());
        certificate.setDrawingNumber(bizProductionPlanDetail.getDrawingNumber());
        certificate.setSpecification(bizProductionPlanDetail.getSpecification());
        certificate.setProductDescription(bizProductionPlanDetail.getProductDescription());
        certificate.setProductionPlanDetailId(bizProductionPlanDetail.getId());
        certificate.setProductId(bizDaywork.getProductId());
        long result = bizDaywork.getTemporaryProcessQualifiedNum() - storageNum + retrievalNum;
        certificate.setNumber((int)result);
        certificate.setPackageNumber(bizDaywork.getTemporaryProcessQualifiedNum());
        certificate.setTenantId(getLoginUser().getTenantId());
        certificate.setPrintDate(new Date());
        certificate.setFactoryName("德迈仕");
        if (bizDaywork.getPackageResourceGroupId() != null && !bizDaywork.getPackageResourceGroupId().equals(0L)) {
            BizPackageResourceGroup bizPackageResourceGroup = bizPackageResourceGroupService.getById(bizDaywork.getPackageResourceGroupId());
            SysDept dept = sysDeptService.selectDeptById(bizPackageResourceGroup.getDeptId());
            certificate.setDeptName(dept.getDeptName());
            certificate.setDeptId(bizPackageResourceGroup.getDeptId());
            certificate.setPackageLineNo(bizPackageResourceGroup.getPackageLineNo().toString());
            certificate.setPackageResourceGroupId(bizPackageResourceGroup.getId());
            certificate.setPackageResourceGroupName(bizPackageResourceGroup.getName());
            certificate.setDeptNo(bizPackageResourceGroup.getDeptNo().toString());
            certificate.setYear(bizPackageResourceGroup.getYear());
        }
        BizCompanyProductPackage packageConditions = new BizCompanyProductPackage();
        packageConditions.setProductId(certificate.getProductId());
        packageConditions.setCompanyId(certificate.getCompanyId());
        List<BizCompanyProductPackage> bizCompanyProductPackages = bizCompanyProductPackageService.getList(packageConditions);
        if (bizCompanyProductPackages.size() == 1) {
            certificate.setPackageCode(bizCompanyProductPackages.get(0).getPackageCode());
            certificate.setPackageStandard(bizCompanyProductPackages.get(0).getPackageStandard());
            certificate.setPackageId(bizCompanyProductPackages.get(0).getAccessoriesId());
            certificate.setPackageTypeCode(bizCompanyProductPackages.get(0).getPackageTypeCode());
        }
        return success(certificate);
    }

    @Log(title = "复制打印合格证", businessType = BusinessType.SELECT)
    @PostMapping("/copyCertificate")
    public AjaxResult copyCertificate(@RequestBody BizCertificate bizCertificate) {
        BizCertificate newCertificate = new BizCertificate(bizCertificate);
        newCertificate.setId(snowflakeIdWorker.nextId());
        return success(newCertificate);
    }

}
