package cn.ezhizao.project.business.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.domain.BizDayworkQualified;
import cn.ezhizao.project.business.service.IBizDayworkQualifiedService;


/**
 * 报公合格数统计Controller
 *
 * @author ezhizao
 * @date 2024-03-08
 */
@RestController
@RequestMapping("/business/qualified")
public class BizDayworkQualifiedController extends BaseController
{
    @Resource
    private IBizDayworkQualifiedService bizDayworkQualifiedService;

    /**
     * 查询报公合格数统计列表
     */
    @PreAuthorize("@ss.hasPermi('business:qualified:list')")
    @GetMapping("/list")
    @Log(title = "查询报公合格数统计列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizDayworkQualified bizDayworkQualified) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkQualified);
        startPage();
        List<BizDayworkQualified> list = bizDayworkQualifiedService.getList(bizDayworkQualified);
        return getDataTable(list);
    }

    /**
     * 导出报公合格数统计列表
     */
    @PreAuthorize("@ss.hasPermi('business:qualified:export')")
    @Log(title = "导出报公合格数统计列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizDayworkQualified bizDayworkQualified) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkQualified);
        List<BizDayworkQualified> list = bizDayworkQualifiedService.getList(bizDayworkQualified);
        ExcelUtil<BizDayworkQualified> util = new ExcelUtil<BizDayworkQualified>(BizDayworkQualified.class);
        util.exportExcel(response, list, "报公合格数统计数据");
    }

    /**
     * 获取报公合格数统计详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:qualified:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取报公合格数统计详细信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizDayworkQualifiedService.getById(id));
    }

    /**
     * 新增报公合格数统计
     */
    @PreAuthorize("@ss.hasPermi('business:qualified:add')")
    @Log(title = "新增报公合格数统计", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizDayworkQualified bizDayworkQualified) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkQualified);
        return toAjax(bizDayworkQualifiedService.save(bizDayworkQualified));
    }

    /**
     * 修改报公合格数统计
     */
    @PreAuthorize("@ss.hasPermi('business:qualified:edit')")
    @Log(title = "修改报公合格数统计", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizDayworkQualified bizDayworkQualified) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkQualified);
        return toAjax(bizDayworkQualifiedService.updateById(bizDayworkQualified));
    }

    /**
     * 删除报公合格数统计
     */
    @PreAuthorize("@ss.hasPermi('business:qualified:remove')")
    @Log(title = "删除报公合格数统计", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizDayworkQualifiedService.removeBatchByIds(ids));
    }
}
