package cn.ezhizao.project.business.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.domain.BizInboundOrderDetail;
import cn.ezhizao.project.business.service.IBizInboundOrderDetailService;

/**
 * 生产入库明细Controller
 *
 * @author ezhizao
 * @date 2025-05-20
 */
@RestController
@RequestMapping("/business/inboundOrderDetail")
public class BizInboundOrderDetailController extends BaseController
{
    @Resource
    private IBizInboundOrderDetailService bizInboundOrderDetailService;

    /**
     * 查询生产入库明细列表
     */
    @PreAuthorize("@ss.hasPermi('business:inboundOrderDetail:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizInboundOrderDetail bizInboundOrderDetail) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInboundOrderDetail);
        startPage();
        List<BizInboundOrderDetail> list = bizInboundOrderDetailService.getList(bizInboundOrderDetail);
        return getDataTable(list);
    }

    /**
     * 导出生产入库明细列表
     */
    @PreAuthorize("@ss.hasPermi('business:inboundOrderDetail:export')")
    @Log(title = "生产入库明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizInboundOrderDetail bizInboundOrderDetail) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInboundOrderDetail);
        List<BizInboundOrderDetail> list = bizInboundOrderDetailService.getList(bizInboundOrderDetail);
        ExcelUtil<BizInboundOrderDetail> util = new ExcelUtil<BizInboundOrderDetail>(BizInboundOrderDetail.class);
        util.exportExcel(response, list, "生产入库明细数据");
    }

    /**
     * 获取生产入库明细详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:inboundOrderDetail:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizInboundOrderDetailService.getById(id));
    }

    /**
     * 新增生产入库明细
     */
    @PreAuthorize("@ss.hasPermi('business:inboundOrderDetail:add')")
    @Log(title = "生产入库明细", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizInboundOrderDetail bizInboundOrderDetail) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInboundOrderDetail);
        return toAjax(bizInboundOrderDetailService.save(bizInboundOrderDetail));
    }

    /**
     * 修改生产入库明细
     */
    @PreAuthorize("@ss.hasPermi('business:inboundOrderDetail:edit')")
    @Log(title = "生产入库明细", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizInboundOrderDetail bizInboundOrderDetail) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInboundOrderDetail);
        return toAjax(bizInboundOrderDetailService.updateById(bizInboundOrderDetail));
    }

    /**
     * 删除生产入库明细
     */
    @PreAuthorize("@ss.hasPermi('business:inboundOrderDetail:remove')")
    @Log(title = "生产入库明细", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizInboundOrderDetailService.removeBatchByIds(ids));
    }
}
