package cn.ezhizao.project.business.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.config.SecurityConfig;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.service.*;
import cn.ezhizao.project.system.service.ISysDeptService;

import lombok.Data;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 修改后工艺Controller
 *
 * @author ezhizao
 * @date 2024-04-07
 */
@RestController
@RequestMapping("/business/amend")
public class BizLotTechnologicalProcessController extends BaseController {
    @Resource
    private IBizLotTechnologicalProcessService bizLotTechnologicalProcessServic;

    @Resource
    private IBizTechnologicalProcessService bizTechnologicalProcessService;

    @Resource
    private IBizLotTechnologicalProcessDetailService bizLotTechnologicalProcessDetailService;

    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;

    @Resource
    private IBizTechnologicalProcessDetailService bizTechnologicalProcessDetailService;

    @Resource
    private IBizLotService bizLotService;

    @Resource
    private IBizProductionPlanDetailService bizProductionPlanDetailService;

    @Resource
    private IBizFurnaceNoInfoService bizFurnaceNoInfoService;

    @Resource
    private IBizDayworkService bizDayworkService;

    @Resource
    private IBizProcessService bizProcessService;

    @Resource
    private ISysDeptService sysDeptService;

    @Resource
    private IBizDayworkCarrierService bizDayworkCarrierService;

    @Resource
    private IBizLotWasteService bizLotWasteService;

    @Resource
    private IBizDayworkItemService bizDayworkItemService;
    @Resource
    private IBizQuickDayworkNextInfoService bizQuickDayworkNextInfoService;
    @Resource
    private IBizOutsourcedOrderDetailService bizOutsourcedOrderDetailService;
    @Resource
    private IBizOutsourcedOrderDetailProcessService bizOutsourcedOrderDetailProcessService;
    @Resource
    private IBizReturnReceiptDetailService bizReturnReceiptDetailService;
    @Resource
    private IBizOutsourceBalanceAccountService bizOutsourceBalanceAccountService;
    @Resource
    private IBizDayworkItemConsultService bizDayworkItemConsultService;
    @Resource
    private IBizProcessInspecionService bizProcessInspecionService;
    @Resource
    private IBizProcessInspectionDetailService bizProcessInspectionDetailService;
    @Resource
    private IBizDayworkAuxiliaryItemService bizDayworkAuxiliaryItemService;
    @Resource
    private IBizDayworkAuxiliaryItemRejectService bizDayworkAuxiliaryItemRejectService;
    @Resource
    private IBizProductionResourceGroupDetailService bizProductionResourceGroupDetailService;
    @Resource
    private IBizNoCurrentRejectService bizNoCurrentRejectService;
    @Resource
    private HttpServletRequest request;

    // 静态内部类 RequestData
    @Data
    public static class RequestData {

        private List<BizLot> lotList;
        private List<BizTechnologicalProcessDetail> processList;

    }
    private static final Object LOCK = new Object();

    /**
     * 查询修改后工艺列表
     */
    @PreAuthorize("@ss.hasPermi('business:amend:list')")
    @GetMapping("/list")
    @Log(title = "查询修改后的工艺列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizLotTechnologicalProcess bizLotTechnologicalProcess) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizLotTechnologicalProcess);
        startPage();
        List<BizLotTechnologicalProcess> list = bizLotTechnologicalProcessServic.getList(bizLotTechnologicalProcess);
        return getDataTable(list);
    }

    /**
     * 查询工序数据
     */
    @GetMapping("/processList")
    @Log(title = "查询工序数据", businessType = BusinessType.SELECT)
    public TableDataInfo processList(BizProcess process) throws NoSuchFieldException, IllegalAccessException {
        startPage();
        return getDataTable(bizProcessService.getList(process));
    }

    /**
     * 查询标准工序数据
     */
    @PreAuthorize("@ss.hasPermi('business:lot:standard')")
    @GetMapping("/standardProcess/{processId}")
    @Log(title = "查询标准工序数据", businessType = BusinessType.SELECT)
    public TableDataInfo standardProcess(@PathVariable("processId") Long processId) throws NoSuchFieldException, IllegalAccessException {
        BizLotTechnologicalProcess amend = bizLotTechnologicalProcessServic.getById(processId);

        return getDataTable(bizTechnologicalProcessDetailService.getProcessDetailsByTechnicalProcessId(amend.getTechnologicalProcessId()));
    }

    /**
     * 查询批次信息
     *
     * @param lotCode
     * @return
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     */
    @GetMapping("/lotParticulars/{lotCode}")
    @Log(title = "查询批次信息", businessType = BusinessType.SELECT)
    public AjaxResult lotParticulars(@PathVariable("lotCode") String lotCode) throws NoSuchFieldException, IllegalAccessException {
        //根据查询批次信息
        BizLot lot = new BizLot();
        lot.setLotCode(lotCode);
        try {
            lot = bizLotService.getList(lot).get(0);
        } catch (Exception e) {
            throw new RuntimeException("批次信息异常！");
        }
        //查询批废
        BizLotWaste lotWaste = bizLotWasteService.query().eq("lot_id", lot.getId()).eq("deleted", 0).one();
        lot.setLotWaste(lotWaste);
        //查询计划单信息
        lot.setProductionPlanDetail(bizProductionPlanDetailService.getById(lot.getProductionPlanDetailId()));
        //查询报工信息
        BizDaywork daywork = new BizDaywork();
        daywork.setLotId(lot.getId());
        List<BizDaywork> dayworks = bizDayworkService.getList(daywork);
        if (dayworks.size() > 0) {
            daywork = bizDayworkService.getList(daywork).get(0);
            lot.setDaywork(daywork);
            //根据报工id和计划单id查询 furnaceNoInfo
            BizFurnaceNoInfo furnaceNoInfo = new BizFurnaceNoInfo();
            furnaceNoInfo.setProductionPlanDetailId(lot.getProductionPlanDetailId());
            furnaceNoInfo.setDayworkId(daywork.getId());

            //查询箱号
            BizDayworkCarrier dayworkCarrier = new BizDayworkCarrier();
            dayworkCarrier.setDayworkId(daywork.getId());
            dayworkCarrier.setIsChanged(0);
            List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.getList(dayworkCarrier);
            StringBuffer carrierName = new StringBuffer();
            dayworkCarriers.forEach(item -> {
                carrierName.append(item.getCarrierCode()).append(", ");
            });
            // 删除末尾多余的逗号和空格
            if (carrierName.length() > 0) {
                carrierName.delete(carrierName.length() - 2, carrierName.length());
            }
            lot.setCarrierName(carrierName.toString());
            //设置furnaceNoInfo
            lot.setFurnaceNoInfo(bizFurnaceNoInfoService.getList(furnaceNoInfo).size() > 0 ? bizFurnaceNoInfoService.getList(furnaceNoInfo).get(0) : new BizFurnaceNoInfo());
            lot.getProductionPlanDetail().setRequisitionDepartmentName(sysDeptService.selectDeptById(lot.getProductionPlanDetail().getRequisitionDepartmentId()).getDeptName());

        } else {
            lot.setFurnaceNoInfo(new BizFurnaceNoInfo());
            lot.getProductionPlanDetail().setRequisitionDepartmentName("");
        }

        //查询工序信息
        lot = selectProcess(lot);

        return success(lot);
    }

    /**
     * 查询工序信息
     */
    @Log(title = "查询工序信息", businessType = BusinessType.SELECT)
    private BizLot selectProcess(BizLot lot) throws NoSuchFieldException, IllegalAccessException {

        BizTechnologicalProcessDetail processDetail = new BizTechnologicalProcessDetail();
        List<BizTechnologicalProcessDetail> processDetailList = new ArrayList<>();

        //是否单批单改
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            //查询更改后的数据
            BizLotTechnologicalProcessDetail amendDetail = new BizLotTechnologicalProcessDetail();
            amendDetail.setLotTechnologicalProcessId(lot.getTechnologicalProcessId());
            List<BizLotTechnologicalProcessDetail> amendDetailList = bizLotTechnologicalProcessDetailService.getList(amendDetail);

            for (BizLotTechnologicalProcessDetail item : amendDetailList) {
                BizTechnologicalProcessDetail process = new BizTechnologicalProcessDetail();
                process.setProcessId(item.getProcessId());
                process.setProcessStepNumber(item.getProcessStepNumber());
                process.setProcessCode(item.getProcessCode());
                process.setProcessAlias(item.getProcessAlias());
                process.setProcessKey(item.getProcessKey());
                process.setProductShaftCategory(item.getProductShaftCategory());
                process.setProductShaftCategoryName(item.getProductShaftCategoryName());
                process.setTechnologicalProcessId(item.getLotTechnologicalProcessId());
                process.setProductId(item.getProductId());
                process.setProductDescription(item.getProductDescription());
                processDetailList.add(process);
            }
        } else {
            //查询工序信息
            processDetail.setTechnologicalProcessId(lot.getTechnologicalProcessId());
            processDetailList = bizTechnologicalProcessDetailService.getProcessDetailsByTechnicalProcessId(lot.getTechnologicalProcessId());

        }

        //查询报工信息
        lot.setTechnologicalProcessDetailList(selectDayworkItem(processDetailList, lot));
        return lot;
    }

    /**
     * 查询报工信息
     *
     * @param processDetailList
     * @param lot
     */
    @Log(title = "查询报工信息", businessType = BusinessType.SELECT)
    private List<BizTechnologicalProcessDetail> selectDayworkItem(List<BizTechnologicalProcessDetail> processDetailList, BizLot lot) {
        //查询报工信息
        BizDayworkItem item = new BizDayworkItem();
        if (lot.getDaywork() != null) {
            item.setDayworkId(lot.getDaywork().getId());
            List<BizDayworkItem> dayworkItems = bizDayworkItemService.selectDayworkItemList(item);

            for (BizTechnologicalProcessDetail t : processDetailList) {
                //给备料赋值，备料没有废品，默认完成
//                if (processDetailList.get(0).getProcessId().equals(t.getProcessId())) {
                if (t.getProcessId() == 1 || t.getProcessId() == 2) {
                    t.setInvestNum(lot.getProductionQuantity());
                    t.setAbandonNum(0);
                    t.setQualifiedNum(lot.getProductionQuantity());
                    t.setStatus("2");
                    t.setEndTime(lot.getCreateTime());
                }

                for (BizDayworkItem d : dayworkItems) {
                    //如果是单批单改只查询FormDayworkItemId为0的数据
                    if (lot.getIsAmend() == 1 && d.getFormDayworkItemId() == 0) {
                        continue;
                    }
                    //判断首序投产数是否为0，如果为0设置投产量为批次投产量
//                    if (processDetailList.get(1).getProcessId().equals(t.getProcessId())) {
//                        if(d.getProdNum()==null||d.getProdNum()==0){
//                            d.setProdNum(lot.getProductionQuantity());
//
//                        }
//                    }


                    //根据工序判断，查询每道工序的报工信息
                    if (t.getProcessId().equals(d.getProcessId()) && d.getProcessStepNumber().equals(t.getProcessStepNumber())) {
                        //查询状态为完成货工序已完成的状态，计算合格量，废品，投入量
                        if ((d.getStatus().equals("2") || d.getStatus().equals("3")) && d.getProcessStepNumber().equals(t.getProcessStepNumber())) {
                            if (d.getRejectNum() != null) {
                                t.setAbandonNum(t.getAbandonNum() == null ? d.getRejectNum() : t.getAbandonNum() + d.getRejectNum());
                            } else {
                                t.setAbandonNum(t.getAbandonNum() == null ? 0 : t.getAbandonNum() != 0 ? t.getAbandonNum() : 0);
                            }
                            t.setQualifiedNum(t.getQualifiedNum() == null ? d.getQualifiedNum() : t.getQualifiedNum() + d.getQualifiedNum());
                            t.setInvestNum(d.getProdNum());
                        }
                        //未开始时合格量默，废品默认为0
                        if (d.getStatus().equals("0")) {
                            t.setAbandonNum(0);
                            t.setQualifiedNum(0);
                            t.setInvestNum(d.getProdNum());
                            //设置状态为未开始
                            t.setStatus("0");
                        } else if (d.getStatus().equals("1") || d.getStatus().equals("2")) {
                            //设置状态为进行中
                            t.setInvestNum(d.getProdNum());
                            t.setStatus("1");
                        } else if (d.getStatus().equals("3")) {
                            t.setStatus("2");
                            t.setEndTime(d.getEndTime());
                        } else {
                            //设置状态为已完成
                            t.setStatus("2");
                        }
                        //设置当前批次状态，当第一道序（不包括备料）状态为未开始时批次状态为未开始
                        if (processDetailList.get(1).getProcessId().equals(t.getProcessId()) && d.getStatus().equals("0")) {
                            lot.getDaywork().setStatus(0);
                        } else if (processDetailList.get(processDetailList.size() - 1).getProcessId().equals(d.getProcessId())) {
                            //最后一道工序如果状态为不是"未开始，进行中，已完成"当前批次已完成
                            if (!d.getStatus().equals("1") && !d.getStatus().equals("2") && !d.getStatus().equals("0")) {
                                lot.getDaywork().setStatus(2);
                            }
                        } else {
                            //当前批次进行中
                            lot.getDaywork().setStatus(1);
                        }
                    }
                }
            }
        }
        return processDetailList;
    }

    /**
     * 导出修改后工艺列表
     */
    @PreAuthorize("@ss.hasPermi('business:amend:export')")
    @Log(title = "导出修改后工艺列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizLotTechnologicalProcess bizLotTechnologicalProcess) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizLotTechnologicalProcess);
        List<BizLotTechnologicalProcess> list = bizLotTechnologicalProcessServic.getList(bizLotTechnologicalProcess);
        ExcelUtil<BizLotTechnologicalProcess> util = new ExcelUtil<BizLotTechnologicalProcess>(BizLotTechnologicalProcess.class);
        util.exportExcel(response, list, "修改后工艺数据");
    }


    @PreAuthorize("@ss.hasPermi('business:lot:multiSingleChange')")
    @Log(title = "多批单改", businessType = BusinessType.INSERT)
    @PostMapping("/addMultiAmend")
    @Transactional
    public AjaxResult addMultiAmend(@RequestBody RequestData requestData) {
        synchronized (LOCK) {
            List<BizLot> lotList = requestData.getLotList();
            List<BizLot> newLotList = new ArrayList<>();
            List<BizTechnologicalProcessDetail> processDetailList = requestData.getProcessList();
            for (BizLot item : lotList) {
                BizLot lot = new BizLot();
                lot.setLotCode(item.getLotCode());
                lot = bizLotService.getList(lot).get(0);

                BizTechnologicalProcess technologicalProcess = new BizTechnologicalProcess();
                //新增修改后工艺信息
                BizLotTechnologicalProcess amend = new BizLotTechnologicalProcess();
                amend.setTenantId(lot.getTenantId());
                amend.setLotId(lot.getId());
                amend.setLotCode(lot.getLotCode());
                amend.setProductionPlanId(lot.getProductionPlanId());
                amend.setProductionPlanDetailId(lot.getProductionPlanDetailId());
                amend.setProductId(lot.getProductId());
                amend.setAmendmentVersions(1);
                amend.setTechnologicalProcessId(lot.getTechnologicalProcessId());
                amend.setTechnologyVersion(lot.getTechnologyVersion());
                amend.setProductDrawingNumber(technologicalProcess.getProductDrawingNumber());
                amend.setProductShaftCategory(technologicalProcess.getProductShaftCategory());
                amend.setProductSpecification(technologicalProcess.getProductSpecification());
                amend.setProductShaftCategoryName(technologicalProcess.getProductShaftCategoryName());
                amend.setProductCode(technologicalProcess.getProductCode());
                amend.setProductDescription(technologicalProcess.getProductDescription());
                bizLotTechnologicalProcessServic.save(amend);

                List<BizLotTechnologicalProcessDetail> amendDetailList = new ArrayList<>();

                for (BizTechnologicalProcessDetail detail : processDetailList) {
                    BizLotTechnologicalProcessDetail amendDetail = new BizLotTechnologicalProcessDetail();
                    amendDetail.setLotTechnologicalProcessId(amend.getId());
                    amendDetail.setTenantId(amend.getTenantId());
                    amendDetail.setTechnologyVersion(amend.getTechnologyVersion());
                    amendDetail.setProductId(amend.getProductId());
                    amendDetail.setProductCode(amend.getProductCode());
                    amendDetail.setProcessStepNumber(detail.getProcessStepNumber());
                    amendDetail.setIsConceal(detail.getIsConceal());
                    amendDetail.setLotId(lot.getId());
                    amendDetail.setProcessId(detail.getProcessId());
                    amendDetail.setProcessKey(detail.getProcessKey());
                    amendDetail.setProcessCode(detail.getProcessCode());
                    amendDetail.setProcessAlias(detail.getProcessAlias());
                    amendDetail.setProductShaftCategory(amend.getProductShaftCategory());
                    amendDetail.setProductShaftCategoryName(amend.getProductShaftCategoryName());
                    amendDetail.setProductDescription(amend.getProductDescription());
                    amendDetail.setTenantId(detail.getTenantId() != null ? detail.getTenantId() : Long.parseLong(request.getHeader("tenantId")));
                    amendDetailList.add(amendDetail);
                }
                bizLotTechnologicalProcessDetailService.saveBatch(amendDetailList);
                updateDayworkItem(lot.getId(), amend.getId(), amendDetailList);

                //修改批次工艺信息
                lot.setTechnologicalProcessId(amend.getId());
                lot.setIsAmend(1);
                newLotList.add(lot);
            }
            return success(bizLotService.saveOrUpdateBatch(newLotList));
        }
    }

    /**
     * 新增修改后工艺
     */
    @PreAuthorize("@ss.hasPermi('business:lot:onceSingleChange')")
    @Log(title = "单批单改", businessType = BusinessType.INSERT)
    @PostMapping("/addAmend/{lotId}")
    @Transactional
    public AjaxResult addAmend(@PathVariable("lotId") Long lotId, @RequestBody List<BizTechnologicalProcessDetail> detailList) {
        //根据id查询批次信息
        synchronized (LOCK) {
            BizLot lot = bizLotService.getById(lotId);
            BizTechnologicalProcess technologicalProcess = new BizTechnologicalProcess();
            BizLotTechnologicalProcess lotTechnologicalProcess = new BizLotTechnologicalProcess();
            int num = 0;//修改过的次数

            //是否进行过单批单改
            if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
                //查询单批单改信息
                BizLotTechnologicalProcess technologicalAmend = bizLotTechnologicalProcessServic.getById(lot.getTechnologicalProcessId());

                //先将lot的工艺id设置为原始工艺id,为了给新改的原始工艺赋值
                lot.setTechnologicalProcessId(technologicalAmend.getTechnologicalProcessId());
                //查询修改过几次
                List<BizLotTechnologicalProcess> list = bizLotTechnologicalProcessServic.selectUpdateList(lot.getId());
                num = list.size();
                //设为停用
                technologicalAmend.setIsStop(1);
                bizLotTechnologicalProcessServic.saveOrUpdate(technologicalAmend);
                //查询原始工艺信息
//            technologicalProcess = bizTechnologicalProcessService.getById(technologicalAmend.getTechnologicalProcessId());
                lotTechnologicalProcess = bizLotTechnologicalProcessServic.query().eq("id", technologicalAmend.getId()).one();
            } else {
                technologicalProcess = bizTechnologicalProcessService.getById(lot.getTechnologicalProcessId());
            }


            //新增修改后工艺信息
            BizLotTechnologicalProcess amend = new BizLotTechnologicalProcess();
            amend.setTenantId(lot.getTenantId());
            amend.setLotId(lot.getId());
            amend.setLotCode(lot.getLotCode());
            amend.setProductionPlanId(lot.getProductionPlanId());
            amend.setProductionPlanDetailId(lot.getProductionPlanDetailId());
            amend.setProductId(lot.getProductId());
            amend.setAmendmentVersions(num + 1);//设置版本号为修改次数+1
            amend.setTechnologicalProcessId(lot.getTechnologicalProcessId());
            amend.setTechnologyVersion(lot.getTechnologyVersion());
            if (lot.getIsAmend() != 1 && lot.getIsWasteRecycling() != 1) {
                amend.setProductDrawingNumber(technologicalProcess.getProductDrawingNumber());
                amend.setProductShaftCategory(technologicalProcess.getProductShaftCategory());
                amend.setProductSpecification(technologicalProcess.getProductSpecification());
                amend.setProductShaftCategoryName(technologicalProcess.getProductShaftCategoryName());
                amend.setProductCode(technologicalProcess.getProductCode());
                amend.setProductDescription(technologicalProcess.getProductDescription());
            } else {
                amend.setProductDrawingNumber(lotTechnologicalProcess.getProductDrawingNumber());
                amend.setProductShaftCategory(lotTechnologicalProcess.getProductShaftCategory());
                amend.setProductSpecification(lotTechnologicalProcess.getProductSpecification());
                amend.setProductShaftCategoryName(lotTechnologicalProcess.getProductShaftCategoryName());
                amend.setProductCode(lotTechnologicalProcess.getProductCode());
                amend.setProductDescription(lotTechnologicalProcess.getProductDescription());
            }
            bizLotTechnologicalProcessServic.save(amend);

            List<BizLotTechnologicalProcessDetail> amendDetailList = new ArrayList<>();

            for (BizTechnologicalProcessDetail detail : detailList) {
                BizLotTechnologicalProcessDetail amendDetail = new BizLotTechnologicalProcessDetail();
                amendDetail.setLotTechnologicalProcessId(amend.getId());
                amendDetail.setTenantId(amend.getTenantId());
                amendDetail.setTechnologyVersion(amend.getTechnologyVersion());
                amendDetail.setProductId(amend.getProductId());
                amendDetail.setIsConceal(detail.getIsConceal());
                amendDetail.setProductCode(amend.getProductCode());
                amendDetail.setProcessStepNumber(detail.getProcessStepNumber());
                amendDetail.setProcessId(detail.getProcessId());
                amendDetail.setLotId(lot.getId());
                amendDetail.setProcessKey(detail.getProcessKey());
                amendDetail.setProcessCode(detail.getProcessCode());
                amendDetail.setProcessAlias(detail.getProcessAlias());
                amendDetail.setProductShaftCategory(amend.getProductShaftCategory());
                amendDetail.setProductShaftCategoryName(amend.getProductShaftCategoryName());
                amendDetail.setProductDescription(amend.getProductDescription());
                amendDetail.setTenantId(detail.getTenantId() != null ? detail.getTenantId() : Long.parseLong(request.getHeader("tenantId")));
                amendDetailList.add(amendDetail);
            }
            bizLotTechnologicalProcessDetailService.saveBatch(amendDetailList);
            updateDayworkItem(lotId, amend.getId(), amendDetailList);

            //修改批次工艺信息value = 1783743753998090243
            lot.setTechnologicalProcessId(amend.getId());
            lot.setIsAmend(1);

            return success(bizLotService.updateById(lot));
        }
    }

    /**
     * 修改报工信息，将原先的报工信息复制出来
     *
     * @param lotId
     */
    @Log(title = "修改报工信", businessType = BusinessType.UPDATE)
    private void updateDayworkItem(Long lotId, Long amendId, List<BizLotTechnologicalProcessDetail> amendDetailList) {
        //查询Daywork数据
        BizDaywork daywork = bizDayworkService.query().eq("lot_id", lotId).one();
        if (daywork != null) {
            // 获取所有报工信息
            List<BizDayworkItem> dayworkItemList = bizDayworkItemService.getListForLotTechnological(new BizDayworkItem().setDayworkId(daywork.getId()));
            if (daywork.getIsAmend() == 1) {
                dayworkItemList = dayworkItemList.stream().filter(v -> !v.getFormDayworkItemId().equals(0L)).collect(Collectors.toList());
            }
            //设置批次工艺信息
            daywork.setTechnologicalProcessId(amendId);
            daywork.setIsAmend(1);
            bizDayworkService.saveOrUpdate(daywork);

            List<BizDayworkItem> list = dayworkItemList.stream().filter(v -> !v.getStatus().equals("0")).collect(Collectors.toList());
            if (list.size() == 0) {
                daywork.setDeleted(1);
                daywork.setRemark("单批单改前未开始加工删除");
                BizDayworkCarrier carrier = new BizDayworkCarrier();
                carrier.setDayworkId(daywork.getId());
                List<BizDayworkCarrier> item = bizDayworkCarrierService.getList(carrier);
                if (item.size() > 0) {
                    item.forEach(v -> {
                        v.setIsChanged(1);
                    });
                }
                bizDayworkCarrierService.saveOrUpdateBatch(item);
                bizDayworkService.saveOrUpdate(daywork);
                bizDayworkService.removeById(daywork.getId());
                bizDayworkItemService.removeBatchByIds(dayworkItemList);
            }


            //修改报工
            //查询DayworkItem数据

            //新增DayworkItem数据
            List<BizDayworkItem> newDayworkItemList = new ArrayList<>();
            dayworkItemList.forEach(v -> {
                amendDetailList.forEach(l -> {
                    if (v.getProcessStepNumber().equals(l.getProcessStepNumber())) {
                        v.setTechnologicalProcessDetailId(l.getId());
                    }
                });
            });

            for (BizDayworkItem dayworkItem : dayworkItemList) {
                //查询工序信息
                if (!dayworkItem.getStatus().equals("0")) {
                    BizDayworkItem newDayworkItem = new BizDayworkItem(dayworkItem);
                    newDayworkItem.setFormDayworkItemId(dayworkItem.getId());
                    newDayworkItem.setProcessStepNumber(dayworkItem.getProcessStepNumber());
                    newDayworkItem.setCreateTime(dayworkItem.getCreateTime());
                    newDayworkItem.setPlace(dayworkItem.getPlace());
                    newDayworkItem.setPlaceId(dayworkItem.getPlaceId());
                    newDayworkItem.setTechnologicalProcessId(amendId);
                    newDayworkItem.setSpecialIsFirst(dayworkItem.getSpecialIsFirst());
                    newDayworkItem.setIsSpecial(dayworkItem.getIsSpecial());
                    newDayworkItem.setSpecialIsFirst(dayworkItem.getSpecialIsFirst());
                    newDayworkItem.setIsWx(dayworkItem.getIsWx());
                    newDayworkItem.setOutsourceDetailId(dayworkItem.getOutsourceDetailId());
                    newDayworkItem.setCarrierNum(dayworkItem.getCarrierNum());

                    newDayworkItem.setIsSort(dayworkItem.getIsSort());
                    newDayworkItem.setIsQuick(dayworkItem.getIsQuick());
                    newDayworkItem.setId(null);

                    //新增报工信息
                    newDayworkItemList.add(newDayworkItem);
                }
            }
            if (newDayworkItemList.size() > 0) {
                //插入报工信息
                bizDayworkItemService.saveBatch(newDayworkItemList);
                bizDayworkItemService.removeBatchByIds(dayworkItemList);
                //外协发出和收回以及结算
                updateOutsourcedDeliver(newDayworkItemList,dayworkItemList);
                //咨询表
                updateDayworkConsult(newDayworkItemList,dayworkItemList);
                //检查表
                updateInspection(newDayworkItemList,dayworkItemList);
                //辅助工序
                updateAuxiliaryDaywork(newDayworkItemList,dayworkItemList);
                //修改正常报工废品
                updateDayworkItemReject(newDayworkItemList,dayworkItemList);
                //资源组明细表
                updateDayworkResourceGroupDetail(newDayworkItemList,dayworkItemList);
                //非本序废品表
                updateDayworkItemNoCurrentReject(newDayworkItemList,dayworkItemList);
                // 快速报工用复制信息
                List<BizQuickDayworkNextInfo> oldQuick = bizQuickDayworkNextInfoService.query().in("daywork_item_id", dayworkItemList.stream().map(BizDayworkItem::getId).collect(Collectors.toList())).list();
                List<BizQuickDayworkNextInfo> newQuick = new ArrayList<>();
                newDayworkItemList.forEach(l -> {
                    if (l.getIsQuick().equals(1) ) {
                        BizQuickDayworkNextInfo old = oldQuick.stream().filter(v -> v.getDayworkItemId().equals(l.getFormDayworkItemId())).findFirst().orElse(null);
                        if (old != null) {
                            BizQuickDayworkNextInfo bizQuickDayworkNextInfo = new BizQuickDayworkNextInfo(old);
                            bizQuickDayworkNextInfo.setDayworkItemId(l.getId());
                            newQuick.add(bizQuickDayworkNextInfo);
                        }
                    }
                });
                bizQuickDayworkNextInfoService.removeBatchByIds(oldQuick);
                bizQuickDayworkNextInfoService.saveBatch(newQuick);
            }


//            //修改废品信息记录的daywork_item_id
//            List<BizDayworkItemReject> reject = bizDayworkItemRejectService.query().eq("daywork_id", daywork.getId()).eq("deleted", 0).list();
////            List<Long> ids = reject.stream().map(BizDayworkItemReject::getId).collect(Collectors.toList());
//            if (reject.size() > 0) {
//                bizDayworkItemRejectService.updateDayworkItemId(reject);
//            }
        }
    }
    //外协发出，收回，结算方法
    public void updateOutsourcedDeliver(List<BizDayworkItem> newList, List<BizDayworkItem> oldList) {
        List<BizLotTechnologicalProcessDetail> lotTechnologicalProcessDetailList = bizLotTechnologicalProcessDetailService.query().eq("lot_technological_process_id", newList.get(0).getTechnologicalProcessId()).list();
        //修改outsourceDetail以及outsourceDetailProcess，returnReceiptDetail,outsourceBalanceAccount的数据
        //查询旧的数据时候否存在发往外协的outsourcedDetailId，修改outsourcedDetail表的工艺id，以及outsourcedDetailProcess表的工艺technologicalProcessDetailId
        //发往外协的序
        List<BizDayworkItem> outsourceItemList = oldList.stream().filter(v -> v.getOutsourceDetailId() != 0).collect(Collectors.toList());
        if(!outsourceItemList.isEmpty()){
            List<Long> outsourcedDetailIds = outsourceItemList.stream().map(BizDayworkItem::getOutsourceDetailId).collect(Collectors.toList());
            List<BizOutsourcedOrderDetail> outsourcedDetailList = bizOutsourcedOrderDetailService.query().in("id", outsourcedDetailIds).list();
            outsourcedDetailList.forEach(v->{
                v.setTechnologicalProcessId(newList.get(0).getTechnologicalProcessId());
            });
            bizOutsourcedOrderDetailService.updateBatchById(outsourcedDetailList);
            //外协发出工序
            List<BizOutsourcedOrderDetailProcess> outsourecdOrderDetailProcessList = bizOutsourcedOrderDetailProcessService.query().in("detail_id", outsourcedDetailIds).list();
            outsourecdOrderDetailProcessList.forEach(item ->{
                BizLotTechnologicalProcessDetail bizLotTechnologicalProcessDetail = lotTechnologicalProcessDetailList.stream().filter(v -> v.getProcessId().equals(item.getProcessId()) && v.getProcessStepNumber().equals(item.getProcessStepNumber())).findFirst().orElse(null);
                item.setTechnologicalProcessId(bizLotTechnologicalProcessDetail.getLotTechnologicalProcessId());
                    item.setTechnologicalProcessDetailId(bizLotTechnologicalProcessDetail.getId());
            });
            bizOutsourcedOrderDetailProcessService.updateBatchById(outsourecdOrderDetailProcessList);
            //收回明细
            List<BizReturnReceiptDetail> returnReceiptDetailList = bizReturnReceiptDetailService.query().in("outsource_detail_id", outsourcedDetailIds).list();
            if(!returnReceiptDetailList.isEmpty()) {
                returnReceiptDetailList.forEach(item -> {
                    BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber()) && v.getProcessId().equals(item.getProcessId())).findFirst().orElse(new BizDayworkItem());
                    item.setTechnologicalProcessId(bizDayworkItem.getTechnologicalProcessId());
                    item.setTechnologicalProcessDetailId(bizDayworkItem.getTechnologicalProcessDetailId());
                });
                bizReturnReceiptDetailService.updateBatchById(returnReceiptDetailList);
            }
            //外协结算
            List<BizOutsourceBalanceAccount> outsourceBalanceAccountList = bizOutsourceBalanceAccountService.query().in("outsource_order_detail_id", outsourcedDetailIds).list();
            if(!outsourceBalanceAccountList.isEmpty()) {
                outsourceBalanceAccountList.forEach(item -> {
                    BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber()) && v.getProcessId().equals(item.getProcessId())).findFirst().orElse(new BizDayworkItem());
                    item.setDayworkItemId(bizDayworkItem.getId());
                    item.setTechnologicalProcessId(bizDayworkItem.getTechnologicalProcessId());
                    item.setTechnologicalProcessDetailId(bizDayworkItem.getTechnologicalProcessDetailId());
                });
                bizOutsourceBalanceAccountService.updateBatchById(outsourceBalanceAccountList);
            }
        }
    }

    //咨询方法
    public void updateDayworkConsult(List<BizDayworkItem> newList, List<BizDayworkItem> oldList) {
        List<Long> oldDayworkItemIds = oldList.stream().map(BizDayworkItem::getId).collect(Collectors.toList());
        List<BizDayworkItemConsult> dayworkItemConsultList = bizDayworkItemConsultService.query().in("daywork_item_id", oldDayworkItemIds).list();
        if (!dayworkItemConsultList.isEmpty()) {
            dayworkItemConsultList.forEach(item -> {
                BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getFormDayworkItemId().equals(item.getDayworkItemId())).findFirst().orElse(new BizDayworkItem());
                item.setDayworkItemId(bizDayworkItem.getId());
                item.setTechnologicalProcessId(bizDayworkItem.getTechnologicalProcessId());
                item.setTechnologicalProcessDetailId(bizDayworkItem.getTechnologicalProcessDetailId());
            });
                bizDayworkItemConsultService.updateBatchById(dayworkItemConsultList);
        }
    }
    //检查
    public void updateInspection(List<BizDayworkItem> newList, List<BizDayworkItem> oldList) {
        //修改processInspection表和processInspectionDetail表
        List<Long> oldDayworkItemIds = oldList.stream().map(BizDayworkItem::getId).collect(Collectors.toList());
        List<BizProcessInspecion> processInspecionList = bizProcessInspecionService.query().in("daywork_item_id", oldDayworkItemIds).list();
        if (!processInspecionList.isEmpty()) {
            processInspecionList.forEach(item -> {
                BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getFormDayworkItemId().equals(item.getDayworkItemId())).findFirst().orElse(new BizDayworkItem());
                item.setDayworkItemId(bizDayworkItem.getId());
                item.setTechnologicalProcessId(bizDayworkItem.getTechnologicalProcessId());
            });
            bizProcessInspecionService.updateBatchById(processInspecionList);
            List<BizProcessInspectionDetail> processInspectionDetailList = bizProcessInspectionDetailService.query().in("daywork_item_id", oldDayworkItemIds).list();
            if (!processInspectionDetailList.isEmpty()){
                processInspectionDetailList.forEach(item -> {
                    BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getFormDayworkItemId().equals(item.getDayworkItemId())).findFirst().orElse(new BizDayworkItem());
                    item.setDayworkItemId(bizDayworkItem.getId());
                });
                bizProcessInspectionDetailService.updateBatchById(processInspectionDetailList);
            }
        }
    }
    //辅助报工
    public void updateAuxiliaryDaywork(List<BizDayworkItem> newList, List<BizDayworkItem> oldList) {
        //修改biz_daywork_auxiliary_item表以及biz_daywork_auxiliary_item_reject表
        Long dayworkId = newList.get(0).getDayworkId();
        List<BizDayworkAuxiliaryItem> dayworkAuxiliaryItemList = bizDayworkAuxiliaryItemService.query().eq("daywork_id", dayworkId).list();
        if(!dayworkAuxiliaryItemList.isEmpty()){
            dayworkAuxiliaryItemList.forEach(item -> {
                BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber())).findFirst().orElse(new BizDayworkItem());
                item.setTechnologicalProcessId(bizDayworkItem.getTechnologicalProcessId());
                item.setTechnologicalProcessDetailId(bizDayworkItem.getTechnologicalProcessDetailId());
            });
            bizDayworkAuxiliaryItemService.updateBatchById(dayworkAuxiliaryItemList);
        }
        //辅助报工废品
        List<Long> oldDayworkItemIds = oldList.stream().map(BizDayworkItem::getId).collect(Collectors.toList());
        List<BizDayworkAuxiliaryItemReject> dayworkAuxiliaryItemRejectList = bizDayworkAuxiliaryItemRejectService.query().in("daywork_item_id", oldDayworkItemIds).list();
        if(!dayworkAuxiliaryItemRejectList.isEmpty()){
            dayworkAuxiliaryItemRejectList.forEach(item -> {
            BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getFormDayworkItemId().equals(item.getDayworkItemId())).findFirst().orElse(new BizDayworkItem());
            item.setDayworkItemId(bizDayworkItem.getId());
            });
            bizDayworkAuxiliaryItemRejectService.updateBatchById(dayworkAuxiliaryItemRejectList);
            }
    }
    //资源组明细(邀请报工
    public void updateDayworkResourceGroupDetail(List<BizDayworkItem> newList, List<BizDayworkItem> oldList) {
        //修改biz_production_resource_group_detail表
        Long dayworkId = newList.get(0).getDayworkId();
        List<BizProductionResourceGroupDetail> dayworkResourceGroupDetailList = bizProductionResourceGroupDetailService.query().eq("protem_daywork_id", dayworkId).list();
        if(!dayworkResourceGroupDetailList.isEmpty()) {
            dayworkResourceGroupDetailList.forEach(item -> {
                BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProtemProcessStepNumber())).findFirst().orElse(new BizDayworkItem());
                item.setProtemProcessStepNumber(bizDayworkItem.getProcessStepNumber());
            });
            bizProductionResourceGroupDetailService.updateBatchById(dayworkResourceGroupDetailList);
        }
    }
    //非本序废品
    public void updateDayworkItemNoCurrentReject(List<BizDayworkItem> newList, List<BizDayworkItem> oldList) {
        Long dayworkId = newList.get(0).getDayworkId();
        List<BizNoCurrentReject> noCurrentRejectList = bizNoCurrentRejectService.query().eq("daywork_id", dayworkId).list();
        if(!noCurrentRejectList.isEmpty()){
            noCurrentRejectList.forEach(item -> {
                BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber())).findFirst().orElse(new BizDayworkItem());
                BizDayworkItem findDayworkItem = newList.stream().filter(v -> v.getProcessStepNumber().equals(item.getFindProcessStepNumber())).findFirst().orElse(new BizDayworkItem());
                item.setProcessStepNumber(bizDayworkItem.getProcessStepNumber());
                item.setFindProcessStepNumber(findDayworkItem.getProcessStepNumber());
            });
            bizNoCurrentRejectService.updateBatchById(noCurrentRejectList);
        }
    }
    //正常报工废品
    public void updateDayworkItemReject(List<BizDayworkItem> newList, List<BizDayworkItem> oldList) {
        //修改dayworkItemReject表
        List<Long> oldDayworkItemIds = oldList.stream().map(BizDayworkItem::getId).collect(Collectors.toList());
        List<BizDayworkItemReject> dayworkItemRejectList = bizDayworkItemRejectService.query().in("daywork_item_id", oldDayworkItemIds).list();
        if(!dayworkItemRejectList.isEmpty()) {
            dayworkItemRejectList.forEach(item -> {
                BizDayworkItem bizDayworkItem = newList.stream().filter(v -> v.getFormDayworkItemId().equals(item.getDayworkItemId())).findFirst().orElse(new BizDayworkItem());
                item.setDayworkItemId(bizDayworkItem.getId());
            });
            bizDayworkItemRejectService.updateBatchById(dayworkItemRejectList);
        }
    }
    /**
     * 获取修改后工艺详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:amend:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取修改后工艺详细信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizLotTechnologicalProcessServic.getById(id));
    }

    /**
     * 新增修改后工艺
     */
    @PreAuthorize("@ss.hasPermi('business:amend:add')")
    @Log(title = "新增修改后工艺", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizLotTechnologicalProcess bizLotTechnologicalProcess) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizLotTechnologicalProcess);
        return toAjax(bizLotTechnologicalProcessServic.save(bizLotTechnologicalProcess));
    }

    /**
     * 修改修改后工艺
     */
    @PreAuthorize("@ss.hasPermi('business:amend:edit')")
    @Log(title = "修改后工艺", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizLotTechnologicalProcess bizLotTechnologicalProcess) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizLotTechnologicalProcess);
        return toAjax(bizLotTechnologicalProcessServic.updateById(bizLotTechnologicalProcess));
    }

    /**
     * 删除修改后工艺
     */
    @PreAuthorize("@ss.hasPermi('business:amend:remove')")
    @Log(title = "删除修改后工艺", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizLotTechnologicalProcessServic.removeBatchByIds(ids));
    }
}
