package cn.ezhizao.project.business.controller;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.BizDayworkItem;
import cn.ezhizao.project.business.domain.BizDayworkItemReject;
import cn.ezhizao.project.business.domain.BizInspectionInstruction;
import cn.ezhizao.project.business.service.IBizDayworkItemRejectService;
import cn.ezhizao.project.business.service.IBizDayworkItemService;
import cn.ezhizao.project.business.service.IBizInspectionInstructionService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.domain.BizProductInspectionInstruction;
import cn.ezhizao.project.business.service.IBizProductInspectionInstructionService;

/**
 * 产品配置分选标准、检测指导项Controller
 *
 * @author ezhizao
 * @date 2024-04-23
 */
@RestController
@RequestMapping("/business/productInspectionInstruction")
public class BizProductInspectionInstructionController extends BaseController
{
    @Resource
    private IBizProductInspectionInstructionService bizProductInspectionInstructionService;
    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;
    @Resource
    private IBizDayworkItemService bizDayworkItemService;
    @Resource
    private IBizInspectionInstructionService bizInspectionInstructionService;
    @Resource
    HttpServletRequest request;

    /**
     * 查询产品配置分选标准、检测指导项列表
     */
//    @PreAuthorize("@ss.hasPermi('business:productInspectionInstruction:list')")
    @GetMapping("/list")
    @Log(title = "查询产品配置分选标准，检查指导项列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizProductInspectionInstruction bizProductInspectionInstruction) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductInspectionInstruction);
//        startPage();
        List<BizProductInspectionInstruction> list = bizProductInspectionInstructionService.getList(bizProductInspectionInstruction);
        return getDataTable(list);
    }

    @GetMapping("/getAllInspection")
    @Log(title = "获取所有分选检查标准", businessType = BusinessType.SELECT)
    public TableDataInfo getAllInspection ()  {
        TableDataInfo info = new TableDataInfo();
        List< BizInspectionInstruction> bizInspectionInstructions = bizInspectionInstructionService.query().list();
        List<BizProductInspectionInstruction> productInspectionInstructionList = new ArrayList<>();
        bizInspectionInstructions.forEach(l -> {
            BizProductInspectionInstruction item = new BizProductInspectionInstruction();
            item.setInspectionInstructionId(l.getId());
            item.setStandard(l.getStandard());
            item.setType(l.getType());
            productInspectionInstructionList.add(item);
        });
        info.setRows(productInspectionInstructionList);
        return info;
    }

    /**
     * 导出产品配置分选标准、检测指导项列表
     */
//    @PreAuthorize("@ss.hasPermi('business:productInspectionInstruction:export')")
    @Log(title = "导出产品配置分选标准、检测指导项列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProductInspectionInstruction bizProductInspectionInstruction) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductInspectionInstruction);
        List<BizProductInspectionInstruction> list = bizProductInspectionInstructionService.getList(bizProductInspectionInstruction);
        ExcelUtil<BizProductInspectionInstruction> util = new ExcelUtil<BizProductInspectionInstruction>(BizProductInspectionInstruction.class);
        util.exportExcel(response, list, "产品配置分选标准、检测指导项数据");
    }

    /**
     * 获取产品配置分选标准、检测指导项详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:productInspectionInstruction:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取产品分选标准，检查指导书", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizProductInspectionInstructionService.getById(id));
    }

    /**
     * 新增产品配置分选标准、检测指导项
     */
//    @PreAuthorize("@ss.hasPermi('business:productInspectionInstruction:add')")
    @Log(title = "新增产品配置分选标准、检测指导项", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProductInspectionInstruction bizProductInspectionInstruction) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductInspectionInstruction);
        return toAjax(bizProductInspectionInstructionService.save(bizProductInspectionInstruction));
    }

    @Log(title = "保存产品配置分选标准、检测指导项", businessType = BusinessType.INSERT)
    @PostMapping("/saveBatch")
    public AjaxResult saveBatch(@RequestBody List<BizProductInspectionInstruction> bizProductInspectionInstructions) throws NoSuchFieldException, IllegalAccessException
    {
        for (BizProductInspectionInstruction bizProductInspectionInstruction : bizProductInspectionInstructions) {
            Long tenantId = Long.parseLong(request.getHeader("tenantId"));
            bizProductInspectionInstruction.setTenantId(tenantId);
        }
        return toAjax(bizProductInspectionInstructionService.saveBatch(bizProductInspectionInstructions));
    }

    /**
     * 修改产品配置分选标准、检测指导项
     */
//    @PreAuthorize("@ss.hasPermi('business:productInspectionInstruction:edit')")
    @Log(title = "修改产品配置分选标准、检测指导项", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProductInspectionInstruction bizProductInspectionInstruction) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductInspectionInstruction);
        return toAjax(bizProductInspectionInstructionService.updateById(bizProductInspectionInstruction));
    }

    /**
     * 删除产品配置分选标准、检测指导项
     */
//    @PreAuthorize("@ss.hasPermi('business:productInspectionInstruction:remove')")
    @Log(title = "删除产品配置分选标准、检测指导项", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        List<BizProductInspectionInstruction> productInspectionInstructionList = bizProductInspectionInstructionService.query().in("id", ids).list();
        //该检查标准被添加到那些产品下的工序里
        List<Long> productTechnologicalProcessDetailIds = productInspectionInstructionList.stream().map(BizProductInspectionInstruction::getTechnologicalProcessDetailId).collect(Collectors.toList());//
        List<Long> inspectionInstructionIds = productInspectionInstructionList.stream().map(BizProductInspectionInstruction::getInspectionInstructionId).collect(Collectors.toList());
        List<BizDayworkItemReject> dayworkItemRejectList = bizDayworkItemRejectService.query().in("inspection_instruction_id", inspectionInstructionIds).list();
        if(dayworkItemRejectList.isEmpty()) {
            return toAjax(bizProductInspectionInstructionService.removeBatchByIds(ids));
        }
        List<Long> dayworkItemIds = dayworkItemRejectList.stream().map(BizDayworkItemReject::getDayworkItemId).collect(Collectors.toList());
        List<BizDayworkItem> listByDayworkItemIds = bizDayworkItemService.getListByDayworkItemIds(dayworkItemIds);
        //添加进废品表里的工序id
        List<Long> technologicalProcessDetailIds = listByDayworkItemIds.stream().map(BizDayworkItem::getTechnologicalProcessDetailId).collect(Collectors.toList());
        boolean haveCommonElements = productTechnologicalProcessDetailIds.stream()
                .anyMatch(technologicalProcessDetailIds::contains);
        if(haveCommonElements) {
            return AjaxResult.error("该选择项已被使用，不能删除");
        }else{
            return toAjax(bizProductInspectionInstructionService.removeBatchByIds(ids));
        }
    }
}
