package cn.ezhizao.project.business.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.domain.BizProductionResourceGroupSubPlan;
import cn.ezhizao.project.business.service.IBizProductionResourceGroupSubPlanService;

/**
 * 资源组任务分配Controller
 *
 * @author ezhizao
 * @date 2024-01-11
 */
@RestController
@RequestMapping("/business/resourcePlan")
public class BizProductionResourceGroupSubPlanController extends BaseController
{
    @Resource
    private IBizProductionResourceGroupSubPlanService bizProductionResourceGroupSubPlanService;

    /**
     * 查询资源组任务分配列表
     */
    @GetMapping("/list")
    @Log(title = "查询资源组任务分配列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizProductionResourceGroupSubPlan bizProductionResourceGroupSubPlan) throws NoSuchFieldException, IllegalAccessException
    {
        startPage();
        List<BizProductionResourceGroupSubPlan> list = bizProductionResourceGroupSubPlanService.getList(bizProductionResourceGroupSubPlan);
        return getDataTable(list);
    }

    /**
     * 导出资源组任务分配列表
     */
    @Log(title = "导出资源组任务分配列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProductionResourceGroupSubPlan bizProductionResourceGroupSubPlan) throws NoSuchFieldException, IllegalAccessException
    {
        List<BizProductionResourceGroupSubPlan> list = bizProductionResourceGroupSubPlanService.getList(bizProductionResourceGroupSubPlan);
        ExcelUtil<BizProductionResourceGroupSubPlan> util = new ExcelUtil<BizProductionResourceGroupSubPlan>(BizProductionResourceGroupSubPlan.class);
        util.exportExcel(response, list, "资源组任务分配数据");
    }

    /**
     * 获取资源组任务分配详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizProductionResourceGroupSubPlanService.getById(id));
    }

    /**
     * 新增资源组任务分配
     */
    @Log(title = "新增资源组任务分配", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProductionResourceGroupSubPlan bizProductionResourceGroupSubPlan) throws NoSuchFieldException, IllegalAccessException
    {

        return toAjax(bizProductionResourceGroupSubPlanService.save(bizProductionResourceGroupSubPlan));
    }
    /**
     * 新增资源组任务分配
     */
    @Log(title = "新增资源组任务分配", businessType = BusinessType.INSERT)
    @PostMapping("/saveBatch")
    public AjaxResult saveBatch(@RequestBody List<BizProductionResourceGroupSubPlan> bizProductionResourceGroupSubPlans) throws NoSuchFieldException, IllegalAccessException
    {
        for(BizProductionResourceGroupSubPlan item:bizProductionResourceGroupSubPlans) {
            setTenantId(item);
        }
        return toAjax(bizProductionResourceGroupSubPlanService.saveBatch(bizProductionResourceGroupSubPlans));
    }
    /**
     * 修改资源组任务分配
     */
    @Log(title = "修改资源组任务分配", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProductionResourceGroupSubPlan bizProductionResourceGroupSubPlan) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionResourceGroupSubPlan);
        return toAjax(bizProductionResourceGroupSubPlanService.updateById(bizProductionResourceGroupSubPlan));
    }

    /**
     * 删除资源组任务分配
     */
    @Log(title = "删除资源组任务分配", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizProductionResourceGroupSubPlanService.removeBatchByIds(ids));
    }
}
